<?php


namespace Intergo\Providers\MessageManager\Drivers;


use Intergo\Providers\Enums\LoggingTagEnum;
use Intergo\Log\GrayLogHandler;
use Exception;
use Illuminate\Http\Request;

trait RouteeTrait
{
    /**
     * @var
     */
    private $from;

    /**
     * Where infobip will send response
     *
     * @var string
     */
    private $notifyUrl;

    /**
     * @var
     */
    private $callbackData;

    /**
     *
     * @var string Application ID
     */
    private $applicationId;

    /**
     *
     * @var string secret key of application
     */
    private $secret;

    /**
     *
     * @var string URL endpoint
     */
    private $endpoint;

    public function __construct()
    {
        parent::__construct();
        $this->from = 'SMSTO';
        $this->notifyUrl = config('services.routee.notify_url');
    }

    /**
     * @return array
     * @throws Exception
     */
    public function send()
    {
        // Format 1 => MULTI MESSAGE ==> [[to: "+9779856034616", message: "This is test"], [to: "+9779856034617", message: "This is test"]]
        if (!empty($this->messages))
        {
            return $this->sendMessageToRecipient($this->messages);
        }

        // Format 3 => CAMPAIGN MESSAGE ==> to: ["+9779856034616", "+9779856034617"], message: "This is test"
        return $this->sendCampaignToRecipient();
    }

    /**
     * @param $messages
     *
     * @return array
     * @throws Exception
     */
    public function sendMessageToRecipient($messages)
    {
        $response = [];
        foreach ($messages as $message)
        {
            $text = str_replace('  ', ' ', $message['message']);
            $sender = $this->sender ?? $this->from;
            $postFields = [
                'callback' => [
                    'strategy' => 'OnChange',
                    'url' => $this->notifyUrl,
                ],
                'body' => trim($text),
                'to' => $message['to'],
                'from' => $sender,
                'transcode' => $this->isTranscoded,
            ];
            $url = config('services.routee.api_url');

            $providerResponse = $this->sendRequest('POST', $url, $postFields); // SENT|FAILED
            $providerResponse['_id'] = $message['_id'];
            if(!empty($providerResponse['trackingId']))
            {
                $bulkId = $providerResponse['trackingId'];
                $providerResponse['message_id'] = $bulkId; // Success
            }
            $response[$providerResponse['status']][] = $providerResponse;

        }
        return $response;
    }

    public function sendCampaignToRecipient()
    {
        $sender = $this->from;

        $text = str_replace('  ', ' ', $this->message);
        $sender = $sender ? $sender : $this->from;
        $postFields = [
            'callback' => [
                'strategy' => 'OnChange',
                'url' => $this->notifyUrl,
            ],
            'body' => trim($text),
            'to' => implode(',', $this->recipient),
            'from' => $sender,
            'transcode' => $this->isTranscoded,
        ];
        $url = config('services.routee.api_url') . '/campaign';

        $response = $this->sendRequest('POST', $url, $postFields);

        $isError = $this->handleIfError($response);

        return $isError ? [false] : [$response];
    }

    /**
     * Execute request.
     *
     * @param      $method
     * @param      $endPoint
     * @param null $bodyContent
     *
     * @return mixed
     * @throws Exception
     */
    private function sendRequest($method, $endPoint, $bodyContent = null)
    {
        $authorizationToken = $this->getAuthorizationToken();
        if (empty($authorizationToken))
        {
            return [
                'error' => true,
                'reason' => 'Invalid Credentials',
                'data' => null,
                'status' => 'FAILED'
            ];
        }
        $opts = [
            CURLOPT_URL => $endPoint,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => $method,
            CURLOPT_HTTPHEADER => [
                'authorization: Bearer ' . $authorizationToken,
                'content-type: application/json',
            ],
        ];

        if ($bodyContent)
        {
            $opts[CURLOPT_POSTFIELDS] = json_encode($bodyContent);
        }

        $curlSession = curl_init();
        curl_setopt_array($curlSession, $opts);

        $response = curl_exec($curlSession);
        $err = curl_error($curlSession);

        curl_close($curlSession);

        $loggingHanlder = app(GrayLogHandler::class)->setTag(LoggingTagEnum::MESSAGES_SERVICE);

        if ($err)
        {
            $response = [
                'error' => true,
                'reason' => $err,
                'data' => $bodyContent,
                'status' => 'FAILED'
            ];
            $loggingHanlder->info('Routee Error:' . $err);
            return $response;
        }
        $response = json_decode($response, true);
        if(isset($response['developerMessage']))
        {
            $response = [
                'error' => true,
                'reason' => isset($response['properties']) ? implode(' ', $response['properties']) : $response['developerMessage'],
                'data' => $bodyContent,
                'status' => 'FAILED'
            ];

            $loggingHanlder->info('Routee Error:' . $response['developerMessage']);

            return $response;
        }
        $response['status'] = 'SENT-TO-PROVIDER';
        return $response;
    }

    public function prepareResponse($response)
    {
        $response = new \stdClass();


    }

    /**
     * Handle Routee response error if any.
     *
     * @param $response
     *
     * @return bool
     */
    private function handleIfError($response)
    {
        if (isset($response->developerMessage))
        {
            Bugsnag::notifyException(new Exception($response->developerMessage ?? 'There is an error response by Routee.'));

            if (isset($response->errors) && is_array($response->errors))
            {

                foreach ($response->errors as $error)
                {
                    Bugsnag::notifyException(new Exception($error->developerMessage ?? 'There is an error response by Routee.'));
                }
            }

            return true;
        }

        return false;
    }

    /**
     * Receive callbacks from Provider
     *
     * {@inheritdoc}
     */
    public function receiveCallback(Request $request)
    {
        $data = $request->all();
        if (!isset($data['campaignTrackingId']) && !isset($data['messageId']))
        {
            return false;
        }

        return $this->mapper($data);
    }

    /**
     * Get authorization token.
     *
     *
     * @return mixed
     * @throws Exception
     */
    public function getAuthorizationToken()
    {
        $token = base64_encode($this->applicationId . ":" . $this->secret);

        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL => config('services.routee.auth_url'),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => 'grant_type=client_credentials',
            CURLOPT_HTTPHEADER => [
                'authorization: Basic ' . $token,
                'content-type: application/x-www-form-urlencoded',
            ],
        ]);

        $response = curl_exec($curl);
        $err = curl_error($curl);
        curl_close($curl);

        $loggingHanlder = app(GrayLogHandler::class)->setTag(LoggingTagEnum::MESSAGES_SERVICE);

        if ($err)
        {
            $loggingHanlder->info('Routee Error while getting Token: ' . $err);
            return null;
        }

        $response = json_decode($response);

        if (!isset($response->access_token))
        {
            $loggingHanlder->info('Invalid or Credentials Not found! Please add credentials for Routee');
            return null;
        }

        return $response->access_token;
    }

}
